/* 
   COPYRIGHT 2004,7 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - coreMib.c */
/* rdk@krupczak.org */

/* core MIB objects belonging to agent engine, all platforms */

#ifdef WIN32
#include <windows.h>
#include <Iphlpapi.h>
#include <Pdh.h>
#endif

#include <pthread.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>

#ifndef WIN32
#include <strings.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#include <sys/utsname.h>
#endif

#include <stdlib.h>

#include "xmp.h"
#include "socklib.h"
#include "globals.h"
#include "xmp_vars.h"
#include "xmp_session.h"
#include "plugins.h"
#include "pluginexport.h"
#include "xmp_events.h"

/* ************************************************************* */

static char sysDescription[256];
static char sysObjectID[256];
char sysName[256];
static u_int sysServices = 0;

xmpStats_t xmpStats;

static xmpErrorStatus_t var_core();
static xmpErrorStatus_t var_params();
static xmpErrorStatus_t var_setParams();
static xmpErrorStatus_t var_xmpStats();
extern xmpErrorStatus_t var_varObjects();
extern xmpErrorStatus_t var_plugList();
extern xmpErrorStatus_t var_modList();
extern xmpErrorStatus_t var_eventTable();

static xmpVar_t coreVars[] = {

/* core "system" group from MIB-2, RFC 1213*/

{"core", "sysDescr",    displayString,"",1,0,var_core},
{"core", "sysObjectID", displayString,"",2,0,var_core},
{"core", "sysUpTime",   counter,      "",3,0,var_core},
{"core", "sysName",     displayString,"",4,0,var_core},
{"core", "sysServices", integer,      "",5,0,var_core},
{"core", "sysUpTimeStr",displayString,"",6,0,var_core},
{"core", "sysBootTime", displayString,"",7,0,var_core},
{"core", "sysDateTime", displayString,"",8,0,var_core},
{"core", "cartographerUptime", counter, "",9,0,var_core},
{"core", "cartographerUptimeStr", displayString, "",10,0,var_core},
{"core", "xmpdUptime", counter, "",9,0,var_core},
{"core", "xmpdUptimeStr", displayString, "",10,0,var_core},

/* XMP message and protocol statistics */

{"core", "inMessages",  counter,    "",1,0,var_xmpStats},
{"core", "inXmlParseErrs",counter,  "",2,0,var_xmpStats},
{"core", "inNoSuchNames", counter,  "",3,0,var_xmpStats},
{"core", "inBadValues",  counter,   "",4,0,var_xmpStats},
{"core", "inTotalReqVars",counter,  "",5,0,var_xmpStats},
{"core", "inSessions",  counter,    "",6,0,var_xmpStats},
{"core", "inBytes",  counter,       "",7,0,var_xmpStats},
{"core", "outMessages",  counter,   "",8,0,var_xmpStats},
{"core", "outSessions",  counter,   "",9,0,var_xmpStats},
{"core", "outBytes",  counter,      "",10,0,var_xmpStats},
{"core", "outTotalReqVars",  counter,"",11,0,var_xmpStats},

{"core", "inGets", counter,      "",12,0,var_xmpStats},
{"core", "inSets", counter,      "",13,0,var_xmpStats},
{"core", "inResponses", counter, "",14,0,var_xmpStats},
{"core", "inTraps", counter,     "",15,0,var_xmpStats},
{"core", "inSelects", counter,   "",16,0,var_xmpStats},
{"core", "inInserts", counter,   "",17,0,var_xmpStats},
{"core", "inDeletes", counter,   "",18,0,var_xmpStats},
{"core", "inUpdates", counter,   "",19,0,var_xmpStats},

{"core", "outGets", counter,     "",20,0,var_xmpStats},
{"core", "outSets", counter,     "",21,0,var_xmpStats},
{"core", "outResponses", counter,"",22,0,var_xmpStats},
{"core", "outTraps", counter,    "",23,0,var_xmpStats},
{"core", "outSelects", counter,  "",24,0,var_xmpStats},
{"core", "outInserts", counter,  "",25,0,var_xmpStats},
{"core", "outDeletes", counter,  "",26,0,var_xmpStats},
{"core", "outUpdates", counter,  "",27,0,var_xmpStats},

{"core", "inAuthenErrors", counter, "", 28,0,var_xmpStats},
{"core", "outAuthenErrors", counter, "", 29,0,var_xmpStats},
{"core", "inInformation", counter, "", 30,0,var_xmpStats},
{"core", "outInformation", counter, "", 31,0,var_xmpStats},
{"core", "outErrors",counter, "", 32,0,var_xmpStats},

/* config/install/operating parameters */
{"core", "xmpdVersion",  floatingPoint, "",1,0,var_params, NULL, NULL, },
{"core", "xmpdDaemonMode", bool,"",2,0,var_params,var_setParams,NULL, },
{"core", "xmpdTrace",  bool,"", 3, 0, var_params, var_setParams, NULL, },
{"core", "xmpdPort", integer,"", 4, 0, var_params, var_setParams, NULL, },
{"core", "xmpdHomeDir", displayString,"", 5, 0, var_params, NULL, NULL, },
{"core", "xmpdVerbose", integer,"", 6, 0, var_params, var_setParams,NULL, },
{"core", "xmpdConfigFile", displayString,"", 7, 0, var_params, NULL, NULL,},
{"core", "xmpdNumPlugins", gauge,"", 8, 0, var_params, NULL, NULL,},
{"core", "xmpdEventAgeOutInterval",gauge,"",9,0,var_params,var_setParams,NULL},
{"core", "xmpdDefaultManager",displayString,"",10,0,var_params,var_setParams,NULL},
{"core", "xmpdAutoUpgrade",bool,"",11,0,var_params,var_setParams,NULL},

/* supported MIB objects table and columns */
{"core", "varTable", table, "", 0, 0, NULL, NULL, var_varObjects, },
{"core", "varMibName", displayString, "varTable", 1, 0},
{"core", "varObjName", displayString, "varTable", 2, 0},
{"core", "varSyntax", displayString, "varTable", 3, 0},
{"core", "varTableName", displayString, "varTable", 4, 0},

/* loaded-plugins table and columns */
{"core", "pluginTable", table, "", 0, 0, NULL, NULL, var_plugList, },
{"core", "pluginName", displayString, "pluginTable", 1, 0},
{"core", "pluginPathName", displayString, "pluginTable", 2, 0},
{"core", "pluginDescr", displayString, "pluginTable", 3, 0},
{"core", "pluginLoadTime", displayString, "pluginTable", 4, 0},

 /* modules we are tracking for upgrading, pushing, etc. */
{"core", "moduleTable", table, "", 0, 0, NULL, NULL, var_modList, },
{"core", "moduleName", displayString, "moduleTable", 1, 0},
{"core", "moduleVersion", displayString, "moduleTable", 2, 0},
{"core", "moduleOwner", displayString, "moduleTable", 3, 0},
{"core", "moduleDependList", displayString, "moduleTable", 4, 0},
{"core", "moduleType", integer, "moduleTable", 5, 0},
{"core", "moduleContent", displayString, "moduleTable", 6, 0},
{"core", "moduleTime", displayString, "moduleTable", 7, 0},
{"core", "modulePendingVersion", displayString, "moduleTable", 8, 0},

/* events */
{"core", "eventTable", table, "", 0, 0, NULL, NULL, var_eventTable, },
{"core", "eventSource", displayString, "eventTable", 1, 0},
{"core", "eventTimeStamp", displayString, "eventTable", 2, 0},
{"core", "eventType", displayString, "eventTable", 3, 0},
{"core", "eventSysObjectID", displayString, "eventTable", 4, 0},
{"core", "eventAgeOut", gauge, "eventTable", 5, 0},
{"core", "eventPDU", displayString, "eventTable", 6, 0},

};

static int numCoreVars = sizeof(coreVars)/sizeof(xmpVar_t);

/* ************************************************************* */
/* ************************************************************* */
/* ************************************************************* */

#ifdef LINUX

/* ************************************************************* */
/* use the algorithm:
 
   /etc/lsb-release
   /etc/issue
   /etc/issue.net
   /etc/redhat-release
   /etc/slackware-version

   to find out what linux distro we are running on

*/

int what_linux_distro(char distroname[256], int maxdistrolen)
{


  return -1;
}

/* ************************************************************* */
/* system uptime in seconds since boot; function also
   located in sysinfo_linux.c 
*/

u_int64_t getSystemUptime()
{
     FILE *fp;
     char line[1024];
     float aFloat;

     if ((fp = fopen("/proc/uptime","r")) == NULL) {
        perror("open /proc/uptime failed");
        return 0; 
     }

     fgets(line,sizeof(line)-1,fp);
     fclose(fp);

     sscanf(line,"%f",&aFloat);

     return (u_int64_t)aFloat;
}

/* ************************************************************* */
/* our DNS domain determination algorithm:

   /etc/resolv.conf
   getdomainname()
   /etc/defaultdomain
   sysctl() -> /proc/sys/kernel/domainname 
   /etc/sysconfig/network  nisdomain

*/

int get_dns_domain(char *domainname, int maxlen)
{
     char line[1024], token1[64], token2[128];
     FILE *fp;
     char *cptr;

     if ((domainname == NULL) || (maxlen < 5)) {
        return -1;
     }

     if ((fp = fopen("/etc/resolv.conf","r")) != NULL) {
        while (fgets(line,sizeof(line)-1,fp) != NULL) {
          /* strip newline */
          if ((cptr = strchr(line,'\n')) != NULL) { *cptr = 0; }
          sscanf(line,"%s %s",token1,token2);
          if (strcmp(token1,"domain") == 0) {
	     strncpy(domainname,token2,maxlen-1);
             fclose(fp);
             return 1;
          }
        }
        fclose(fp);
     } 

     /* cant seem to find man page on getdomainname() */

     /* newer systems put '(none)' in various places if its not set */

     if ((fp = fopen("/etc/defaultdomain","r")) != NULL) {
        if (fgets(line,sizeof(line)-1,fp) != NULL) {
           if ((cptr = strchr(line,'\n')) != NULL) { *cptr = 0; }
           if (strcmp(line,"(none)") != 0) {
               strncpy(domainname,line,maxlen-1);
               fclose(fp);
               return 1;
           }
        }
        fclose(fp);
     }

     if ((fp = fopen("/proc/sys/kernel/domainname","r")) != NULL) {
        if (fgets(line,sizeof(line)-1,fp) != NULL) {
           if ((cptr = strchr(line,'\n')) != NULL) { *cptr = 0; }
           if (strcmp(line,"(none)") != 0) {
  	      strncpy(domainname,line,maxlen-1);
              fclose(fp);
              return 1;
           }
        }
        fclose(fp);
     }


     if ((fp = fopen("/etc/sysconfig/network","r")) != NULL) {
        while (fgets(line,sizeof(line)-1,fp) != NULL) {
          if ((cptr = strchr(line,'\n')) != NULL) { *cptr = 0; }
          sscanf(line,"%s=%s",token1,token2);
          if (strcmp(token1,"NISDOMAIN") == 0) {
	     strncpy(domainname,token2,maxlen-1);
             fclose(fp);
             return 1;
          }
	}     
        fclose(fp);
     }

     domainname[0] = 0;

     return -1;
}


#endif /* LINUX  */

/* ************************************************************* */
/* ************************************************************* */
/* ************************************************************* */

#ifdef SOLARIS

/* ************************************************************* */
/* system uptime in seconds since boot; for Solaris, we need
   access to kstat functions in order to retrieve this value; for
   now, 
*/

u_int64_t getSystemUptime()
{
     int (*getSystemUptimePtr)();
     u_int64_t up = 0;

     getSystemUptimePtr = (int *)xmpGetSymbolValue("get_uptime");

     if (getSystemUptimePtr != NULL)
        up = (u_int64_t)(getSystemUptimePtr)();
     
     return up;
}

/* ************************************************************* */

int get_dns_domain(char *domainname, int maxlen)
{
     char line[1024], token1[64], token2[128];
     FILE *fp;
     char *cptr;

     if ((domainname == NULL) || (maxlen < 5)) {
        return -1;
     }

     if ((fp = fopen("/etc/resolv.conf","r")) != NULL) {
        while (fgets(line,sizeof(line)-1,fp) != NULL) {
          /* strip newline */
          if ((cptr = strchr(line,'\n')) != NULL) { *cptr = 0; }
          sscanf(line,"%s %s",token1,token2);
          if (strcmp(token1,"domain") == 0) {
	     strncpy(domainname,token2,maxlen-1);
             fclose(fp);
             return 1;
          }
        }
        fclose(fp);
     } 

     /* cant seem to find man page on getdomainname() */

     if ((fp = fopen("/etc/defaultdomain","r")) != NULL) {
        if (fgets(line,sizeof(line)-1,fp) != NULL) {
           if ((cptr = strchr(line,'\n')) != NULL) { *cptr = 0; }
           if (strcmp(line,"(none)") != 0) {
	      strncpy(domainname,line,maxlen-1);
              fclose(fp);
              return 1;
           }
        }
        fclose(fp);
     }

     domainname[0] = 0;

     return -1;
}

#endif /* SOLARIS */

/* ************************************************************* */
/* ************************************************************* */
/* ************************************************************* */

#ifdef WIN32

/* ************************************************************* */
/* system uptime in seconds since boot
   we can get this value via performance registry in
   \\System\\System Up Time 

  we can also use timeGetTime() in winmm.lib if we want to link
  against yet another windows library and introduce yet more bugs and 
  code dependencies

  GetTickCount() returns number of millseconds since system boot but 
  will wrap if system has been up for 49.7 days
*/


u_int64_t getSystemUptime()
{
     static int uptimeInit = 0;
     static HANDLE qh, h1;
     PDH_STATUS retval;
     DWORD counterType;
     u_int64_t up;
     PDH_FMT_COUNTERVALUE counterValue;

     if (uptimeInit == 0) {
        if (PdhOpenQuery(NULL,0,&qh) != ERROR_SUCCESS) {
           fprintf(stderr,"getSystemUptime: fail to open query, %d\n",
                   GetLastError());
           return 0;
        }
        if (PdhAddCounter(qh,"\\System\\System Up Time",0,&h1) 
            != ERROR_SUCCESS) {
           fprintf(stderr,"getsystemUptime: fail to add counter, %lx\n",
                   GetLastError());
           PdhCloseQuery(qh);
           return 0;
        }

        uptimeInit = 1;
     }

     if ((retval = PdhCollectQueryData(qh)) != ERROR_SUCCESS) {
        return 0;
     }

     /* counter is type is 0x30240500 -- PERF_ELAPSED_TIMER 
        use PdhGetFormattedCounterValue() since we have to
        munge the counter in some mysterious way */

     retval = PdhGetFormattedCounterValue(h1,PDH_FMT_LARGE,&counterType,
                                          &counterValue);

     up = (u_int64_t)counterValue.largeValue;

     return up; 
}

/* ************************************************************* */

int get_dns_domain(char *domainname, int maxlen)
{
     FIXED_INFO fi;
     ULONG fisize;

     if (domainname != NULL)
        domainname[0] = 0;

     /* use IP Helper functions new in NT4/SP4+ */
     /* need to install platform SDK to get the include file(s) */
     fisize = sizeof(fi);
     if (GetNetworkParams(&fi,&fisize) < 0)
        return -1;

     strncpy(domainname,fi.DomainName,maxlen-1);
     domainname[maxlen-1] = 0;

     if (xmpd_verbose)
        printf("get_dns_domain: enable:%d '%s'\n",
               fi.EnableDns,domainname);

     if (xmpd_verbose)
         printf("Systemname: '%s'\n",fi.HostName);

     return 0;
}

#endif /* WIN32 */

/* ************************************************************* */
/* ************************************************************* */
/* ************************************************************* */
/* all platforms */

/* ************************************************************* */

int getSystemUptimeStr(char *str, int maxlen)
{
     u_int64_t uptime;
     int days, hours, minutes;

     if ((str == NULL) || (maxlen < 25))
        return 0;

     str[0] = 0;

     uptime = getSystemUptime();

     days = uptime / (24*60*60);

     uptime -= days*(24*60*60);

     hours = uptime / (60*60);

     uptime -= hours*(60*60);

     minutes = uptime / 60;

     sprintf(str,"%d days, %02d:%02d:00",days,hours,minutes);

     return strlen(str);
}

/* get uptime and work backwards */

int getSystemBootTimeStr(char *str, int maxlen)
{
     u_int64_t uptime;
     time_t now;
     char *cptr;

     if ((str == NULL) || (maxlen < 25))
        return 0;

     uptime = getSystemUptime();

     now = time(NULL);

     now -= (time_t)uptime;

     /* convert now to a string */
     str[0] = 0;
     ctime_r(&now,str);

     /* strip off trailing \n */
     if ((cptr = strchr(str,'\n')) != NULL)
        *cptr = 0;

     return strlen(str);
}

int getSystemDateTime(char *str, int maxlen)
{
     time_t now;
     char *cptr;

     now = time(NULL);
     str[0] = 0;

     ctime_r(&now,str);

     /* strip off trailing \n */
     if ((cptr = strchr(str,'\n')) != NULL)
        *cptr = 0;

     return strlen(str);
}

int getCartographerUptime() { return time(NULL) - xmpd_init_time; }

int getCartographerUptimeStr(char *str, int maxlen)
{
     time_t uptime;
     int days, hours, minutes,seconds;

     if ((str == NULL) || (maxlen < 25))
        return 0;

     str[0] = 0;

     /* uptime in seconds */
     uptime = time(NULL) - xmpd_init_time;

     days = uptime / (24*60*60);

     uptime -= days*(24*60*60);

     hours = uptime / (60*60);

     uptime -= hours*(60*60);

     minutes = uptime / 60;

     uptime -= minutes*60;

     seconds = uptime;

     sprintf(str,"%d days, %02d:%02d:%02d",days,hours,minutes,seconds);

     return strlen(str);
}

/* ************************************************************* */
/* fetch value of core managed object */

static xmpErrorStatus_t var_core(xmpVar_t *vp, xmpVariable_t *v)
{
     int ret;

     switch (vp->hint) {
       case 1:
 	  /* description of this system */
 	  ret = sprintf(v->valueStr,"%s",sysDescription);
  	  v->valueStrLen = ret;
          break;
       case 2:
	 /* system/vendor object identifier */
	  ret = sprintf(v->valueStr,"%s",sysObjectID);
  	  v->valueStrLen = ret;
          break;
       case 3:
	  /* sysUpTime */
 	  ret = sprintf(v->valueStr,"%lld",getSystemUptime()); /* lld or ld */
  	  v->valueStrLen = ret;
          break;
       case 4:
	  /* system name */
	  ret = sprintf(v->valueStr,"%s",sysName);
  	  v->valueStrLen = ret;
          break;
       case 5:
	  /* system services */
 	  ret = sprintf(v->valueStr,"%d",sysServices);
  	  v->valueStrLen = ret;
          break;

       case 6:
          ret = getSystemUptimeStr(v->valueStr,sizeof(v->valueStr));
	  v->valueStrLen = ret;
          break;

       case 7: 
          ret = getSystemBootTimeStr(v->valueStr,sizeof(v->valueStr));
	  v->valueStrLen = ret;
          break;

       case 8:
	  ret = getSystemDateTime(v->valueStr,sizeof(v->valueStr));
	  v->valueStrLen = ret;
          break;

       case 9:
	  /* cartographer uptime */
	  ret = sprintf(v->valueStr,"%d",getCartographerUptime());
  	  v->valueStrLen = ret;
          break;

       case 10:
	  /* cartographer uptime string format */
	  ret = getCartographerUptimeStr(v->valueStr,sizeof(v->valueStr));
          v->valueStrLen = ret;
          break;

       default:
  	  v->valueStrLen = 0;
 	  return noSuchName;
     }

     return noError;
}

/* ************************************************************* */
/* set allowable parameters, two-passes */

static xmpErrorStatus_t var_setParams(xmpVar_t *vp,xmpVariable_t *v,int doSet)
{
     int aValue;

     switch (vp->hint) {

       case 2: /* daemonMode */
	  /* test variable value */
	  if ((strcmp(v->valueStr,"true") != 0) && 
              (strcmp(v->valueStr,"false") != 0)) {
	     return badValue;
          }
          if (doSet) {
	     if (strcmp(v->valueStr,"true") == 0)
	        xmpd_daemon = 1;
             else
	        xmpd_daemon = 0;
             xmpd_update_configfile = 1;
          }
	  break;

       case 3: /* trace */
	  /* test variable value */
	  if ((strcmp(v->valueStr,"true") != 0) && 
              (strcmp(v->valueStr,"false") != 0)) {
	     return badValue;
          }
          if (doSet) {
	     if (strcmp(v->valueStr,"true") == 0)
	        xmpd_trace = 1;
             else
	        xmpd_trace = 0;
             xmpd_update_configfile = 1;
          }
	  break;

       case 4: /* port */
	  aValue = atoi(v->valueStr);
          if ((aValue < 1) || (aValue > 65536)) {
	     return badValue;
          }
          if (doSet) {
 	     xmpd_port = (u_short)aValue;
             xmpd_update_configfile = 1;
          }
	  break;

       case 6: /* verbose 0..10 */
	  aValue = atoi(v->valueStr);
          if ((aValue < 0) || (aValue > 10)) {
	     return badValue;
          }
          if (doSet) {
	     xmpd_verbose = aValue;
             xmpd_update_configfile = 1;
          }
	  break;

       case 9: /* age out interval */
	  aValue = atoi(v->valueStr);
          if (aValue < 0) {
	     return badValue;
          }
          if (doSet) {
	     eventAgeOutInterval = aValue;
             xmpd_update_configfile = 1;
          }
          break;

       case 10: /* default manager */
	  if (v->valueStr[0] == 0) {
	     return badValue;
 	  }
          if (doSet) {
	     strncpy(defaultManager,v->valueStr,255);
             defaultManager[255] = 0;
          }
          break;

       case 11: /* autoUpgrade */
	  if ((strcmp(v->valueStr,"true") != 0) && 
              (strcmp(v->valueStr,"false") != 0)) {
	     return badValue;
          }
          if (doSet) {
	     if (strcmp(v->valueStr,"true") == 0)
	        autoUpgrade = 1;
             else
	        autoUpgrade = 0;
             xmpd_update_configfile = 1;
          }
          break;

       default:
  	  v->valueStrLen = 0;
	  return noSuchName;
     }

     return noError;
}

/* ************************************************************* */
/* get xmp stats */

static xmpErrorStatus_t var_xmpStats(xmpVar_t *vp, xmpVariable_t *v)
{
     int ret;

     switch (vp->hint) {
       case 1:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inMessages);
  	  v->valueStrLen = ret;
 	  break;
       case 2:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inXmlParseErrs);
  	  v->valueStrLen = ret;
 	  break;
       case 3:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inNoSuchNames);
  	  v->valueStrLen = ret;
 	  break;
       case 4:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inBadValues);
  	  v->valueStrLen = ret;
 	  break;
       case 5:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inTotalReqVars);
  	  v->valueStrLen = ret;
 	  break;
       case 6:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inSessions);
  	  v->valueStrLen = ret;
 	  break;
       case 7:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inBytes);
  	  v->valueStrLen = ret;
 	  break;
       case 8:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.outMessages);
  	  v->valueStrLen = ret;
 	  break;
       case 9:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.outSessions);
  	  v->valueStrLen = ret;
 	  break;
       case 10:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.outBytes);
  	  v->valueStrLen = ret;
 	  break;
       case 11:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.outTotalReqVars);
  	  v->valueStrLen = ret;
 	  break;

       case 12:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inGets);
  	  v->valueStrLen = ret;
 	  break;
       case 13:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inSets);
  	  v->valueStrLen = ret;
 	  break;
       case 14:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inResponses);
  	  v->valueStrLen = ret;
 	  break;
       case 15:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inTraps);
  	  v->valueStrLen = ret;
 	  break;
       case 16:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inSelects);
  	  v->valueStrLen = ret;
 	  break;
       case 17:
 	  ret = sprintf(v->valueStr,"%llu",xmpStats.inInserts);
  	  v->valueStrLen = ret;
 	  break;
       case 18:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inDeletes);
  	  v->valueStrLen = ret;
 	  break;
       case 19:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.inUpdates);
  	  v->valueStrLen = ret;
 	  break;

       case 20:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.outGets);
  	  v->valueStrLen = ret;
 	  break;
       case 21:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.outSets);
  	  v->valueStrLen = ret;
 	  break;
       case 22:
  	  ret = sprintf(v->valueStr,"%llu",xmpStats.outResponses);
  	  v->valueStrLen = ret;
 	  break;
       case 23:
 	  ret = sprintf(v->valueStr,"%llu",xmpStats.outTraps);
  	  v->valueStrLen = ret;
 	  break;
       case 24:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.outSelects);
  	  v->valueStrLen = ret;
 	  break;
       case 25:
 	  ret = sprintf(v->valueStr,"%llu",xmpStats.outInserts);
  	  v->valueStrLen = ret;
 	  break;
       case 26:
	  ret = sprintf(v->valueStr,"%llu",xmpStats.outDeletes);
  	  v->valueStrLen = ret;
 	  break;
       case 27:
 	  ret = sprintf(v->valueStr,"%llu",xmpStats.outUpdates);
  	  v->valueStrLen = ret;
 	  break;

       case 28:
 	  ret = sprintf(v->valueStr,"%llu",xmpStats.inAuthenErrors);
  	  v->valueStrLen = ret;
 	  break;
       case 29:
 	  ret = sprintf(v->valueStr,"%llu",xmpStats.outAuthenErrors);
  	  v->valueStrLen = ret;
 	  break;
       case 30:
 	  ret = sprintf(v->valueStr,"%llu",xmpStats.inInformation);
  	  v->valueStrLen = ret;
 	  break;
       case 31:
 	  ret = sprintf(v->valueStr,"%llu",xmpStats.outInformation);
  	  v->valueStrLen = ret;
 	  break;
       case 32:
 	  ret = sprintf(v->valueStr,"%llu",xmpStats.outErrors);
  	  v->valueStrLen = ret;
 	  break;

       default:
  	  v->valueStrLen = 0;
 	  return noSuchName;
     }
     return noError;
}

/* ************************************************************* */
/* fetch config/install parameter/values */

static xmpErrorStatus_t var_params(xmpVar_t *vp, xmpVariable_t *v)
{
     int ret;

     if (xmpd_verbose)
        printf("var_params: starting for %s\n",v->varName.objName);
 
     switch (vp->hint) {
       case 1: /* xmpd version */
 	  v->valueStrLen = sprintf(v->valueStr,"%3.2f",xmpd_version);
          break;

       case 2: /* daemon mode, bool */
	  if (xmpd_daemon == 0)
	     ret = sprintf(v->valueStr,"false");
          else 
	     ret = sprintf(v->valueStr,"true");
#ifdef WIN32
          if (win32service) {
             /* if we are running as a service, set our global variable
                 appropriately */
             xmpd_daemon = 1;
	     ret = sprintf(v->valueStr,"true");
          }
#endif
 	  v->valueStrLen = ret;
          break;

       case 3: /* trace */
	  if (xmpd_trace == 0)
	     ret = sprintf(v->valueStr,"false");
          else 
	     ret = sprintf(v->valueStr,"true");
 	  v->valueStrLen = ret;
          break;
       case 4: /* port; if set, bind next time re-start XXX */
 	  v->valueStrLen = sprintf(v->valueStr,"%d",xmpd_port);
          break;
       case 5: /* home-dir */
 	  v->valueStrLen = sprintf(v->valueStr,"%s",xmpd_home_dir);
          break;
       case 6: /* verbose */
	  v->valueStrLen = sprintf(v->valueStr,"%d",xmpd_verbose);
          break;
       case 7: /* config-file */
 	  /* on solaris 64-bit, we cant pass null ptrs to (s)printf for
             strings */
	  if (xmpd_config_file == NULL) {
	     v->valueStrLen = 0;
             v->valueStr[0] = 0;
 	  }
          else 
   	     v->valueStrLen = sprintf(v->valueStr,"%s",xmpd_config_file);
	  break;
       case 8:
	  v->valueStrLen = sprintf(v->valueStr,"%d",getNumberPlugins());
          break;

       case 9: /* age out interval in seconds */
	  v->valueStrLen = sprintf(v->valueStr,"%d",eventAgeOutInterval);
          break;

       case 10: /* default manager */
	  v->valueStrLen = sprintf(v->valueStr,"%s",defaultManager);
          break;

       case 11: /* autoUpgrade */
	  if (autoUpgrade == 1) {
	     ret = sprintf(v->valueStr,"true");
          }
          else {
	     ret = sprintf(v->valueStr,"false");
 	  }
          v->valueStrLen = ret;
          break;

       default:
 	  v->valueStrLen = 0;
          return noSuchName;
     }

     return noError;
}

/* ************************************************************* */

int initCore()
{
     struct utsname uts;
     char dnsdomain[256];
     int i;

     uname(&uts);

     bzero(&xmpStats,sizeof(xmpStats));

     /* sysDescr - osname, os-release, arch */

#ifndef WIN32
     snprintf(sysDescription,255,"%s %s %s",
	     uts.sysname,uts.release,uts.machine);
#else
     _snprintf(sysDescription,255,"%s %s %s",
	      uts.version,uts.release,uts.machine);
#endif

     if (xmpd_verbose)
        printf("sysDescription: %s\n",sysDescription);

     /* sysObjectID -- Krupczak.org, LLC */
     sprintf(sysObjectID,KORGOBJID);

     /* domain-name */
     get_dns_domain(dnsdomain,256);
     if (strcmp(dnsdomain,"(none)") == 0)
        dnsdomain[0] = 0;

     /* sysName -- FQDN */
     sprintf(sysName,"%s",uts.nodename);
     if ((strstr(sysName,dnsdomain) == NULL) && (dnsdomain[0] != 0))
        sprintf(sysName,"%s.%s",uts.nodename,dnsdomain);

     if (xmpd_verbose)
        printf("FQDN: %s\n",sysName);

     /* sysServices -- XXX 
        for now, just a end-to-end host; we should check
        if we are routing, firewalling, running applications, etc. 
      */
     sysServices = 2^(4-1);

     /* sysUpTime is calculated on the fly; sysLocation is 
        near meaningless . . . */

     /* add core MIB objects */
     for (i=0; i<numCoreVars; i++) {
         xmpAddVar(&coreVars[i]);
     }

     initCoreEvents();

     return 1;
}

/* ************************************************************* */
