/* 
   COPYRIGHT 2005 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - modules.h */
/* rdk@krupczak.org */

/* module tracking symbols, structs, etc. */

/* ************************************************************* */
/* main definition of a module, types, etc. */

typedef enum moduleType {
     invalidModuleType = 0,
     agentModule = 1,  /* fetch, stop agent, rename, restart agent */
     pluginModule = 2, /* fetch, unload plugin, rename, reload */
     libModule = 3,    /* fetch, stop agent, rename, restart agent */
     dataModule = 4,   /* fetch, rename, no interruption */
     binModule =  5,   /* fetch, rename, no interruption */
} moduleType_t;

/* data modules: absolute name (e.g. connectionData.xml, ntsetup.exe)
   agent module: xmpd-platform.suffix (.exe or no suffix)
   plugin module: module-platform.suffix (cartographer-win32.dll)
   library module: absolute name (e.g. pthreadVC.dll)
   binary module: name-platform.suffix (xmpquery, restartxmpd)
*/

typedef struct module {
     char modulename[256]; /* the full filename of the module */
     char version[64];     /* monotonically increasing */
     char owner[256];      /* plugin or component that owns it */
     moduleType_t modType; /* type of module */  
     char dependList[256]; /* comma-separate list of module-names */
     time_t timeStamp;     /* last create/modify time */
     char pendingVersion[64]; /* version of any pending upgrade */
} module_t;

int init_modules();
int addModule(char *, char *, char *, char *, moduleType_t);
int removeModule(char *);
int checkPeerModules(char *);

/* ************************************************************* */

