/* 
   COPYRIGHT 2004 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - plugins.h */
/* rdk@krupczak.org */

/* Plugin definitions, APIs, exposed symbols, etc.
   Include this file both within agent and within plugins */

/* ************************************************************* */
/* main definition of a plugin */

typedef struct plugin {

   char modulename[25];            /* well-known name of module */
   char pathname[FILENAME_MAX];    /* where it lives */
   char descr[128];                /* module name/ver/description */

   u_char flags;                   /* always have flags */
#ifndef WIN32
   void *solibhandle;              /* handle for shared-lib */
#else
   HMODULE solibhandle;            /* handle for shared-lib */
#endif

   /* function pointers for invoking module */
#ifndef WIN32
   int    (*init)();               /* initialize library */
   int    (*start)();              /* start/create any threads */
   int    (*finish)();             /* prepare to be unloaded */
   int    (*modinfo)();            /* name, vers/descr */
#else
   FARPROC init;                   /* initialize library */
   FARPROC start;                  /* start/create any threads */
   FARPROC finish;                 /* prepare to be unloaded */
   FARPROC modinfo;                /* name, vers/descr */
#endif

   time_t loadTime;                /* time plugin was loaded */

#if 0   
   struct plugin *next;
#endif

} plugin_t;

/* plugin flags */
#define PLUGIN_EMPTY      0x000
#define PLUGIN_VALID      0x001
#define PLUGIN_UNLOADING  0x002

#define PLUGIN_CLEANUP_TIME   30

int initPlugins();
int findLoadAllPlugins(char *);
int getNumberPlugins();
int unloadAllPlugins();
int cleanupAllPlugins();

char *base64_encode(u_char *, int, int *);
u_char *base64_decode(u_char *, int, int *);

/* ************************************************************* */
/* definitions, symbols, etc. needed by plugins for coding to
   our API spec 
*/

#ifdef PLUGINDEBUG

#define DBG0(s)
#define DBG(s) printf(s);fflush(stdout)
#define DBG1(s,a) printf(s,a);fflush(stdout)
#define DBG2(s,a,b) printf(s,a,b);fflush(stdout)
#define DBG3(s,a,b,c) printf(s,a,b,c);fflush(stdout)
#define DBG4(s,a,b,c,d) printf(s,a,b,c,d);fflush(stdout)
#define DBG5(s,a,b,c,d,e) printf(s,a,b,c,d,e);fflush(stdout)
#define DBG6(s,a,b,c,d,e,f) printf(s,a,b,c,d,e,f);fflush(stdout)

#else

#define DBG0(s)
#define DBG(s)	
#define DBG1(s,a) 	
#define DBG2(s,a,b)
#define DBG3(s,a,b,c)
#define DBG4(s,a,b,c,d)
#define DBG5(s,a,b,c,d,e)
#define DBG6(s,a,b,c,d,e,f)

#endif

/* ************************************************************* */

/* symbols in DLLs need to explicitly be exported so
   we define DLLExport macro to declare them for export;
   on UNIX, we do nothing so DLLExport is blank
   Also defined in xmp.h to get around compiler errors
*/

#ifdef WIN32

#define DLLExport   __declspec( dllexport )
#define DLLImport   __declspec ( dllimport )

#else

/* UNIX */
#define DLLExport
#define DLLImport  

#endif /* WIN32 */

/* ************************************************************* */
