/* 
   COPYRIGHT 2004 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - xmp_vars.h */
/* rdk@krupczak.org */

/* ************************************************************* */
/* internal representation of a MIB object */

typedef struct xmpVar {
     char mibName[128];    /* mib module containing this object/variable */
     char objName[128];    /* the object name */
     xmpSyntax_t syntax;   /* syntax - integer, etc. */
     char tableName[128];  /* tablename, for entries/columns, null otherwise */

     u_int hint;           /* to more easily identify this mib object */
     u_int flags;          /* the dreaded flags */

     /* scalar method routines */
     xmpErrorStatus_t (*get)();   /* get an object value or table entry */
     xmpErrorStatus_t (*set)();   /* set an object value */

     /* tabular method routines */
     xmpErrorStatus_t (*select)();   /* select/get entries */
     xmpErrorStatus_t (*insert)();   /* add/create entries */
     xmpErrorStatus_t (*update)();   /* update/set/modify entries */
     xmpErrorStatus_t (*delete)();   /* delete entries */

     /* linked list fields */
     struct xmpVar *next, *prev;

} xmpVar_t;

xmpVar_t *xmpFindVar(char *, char *);

/* ************************************************************* */
/* internal representation of a MIB container  */

typedef struct xmpMib {

     char mibName[128];

     /* linked list fields */
     struct xmpMib *next, *prev;

    /* MIB objects themselves hang off MIB entry */
    xmpVar_t *xmpVarsHead;
    xmpVar_t *xmpVarsTail;

} xmpMib_t;

xmpMib_t *xmpFindMib(char *);

/* ************************************************************* */

int xmpDumpVarsBackwards();
int xmpDumpVars();

/* ************************************************************* */

