/* 
   COPYRIGHT 2011 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - xmpgetsubgraph.c */
/* rdk@krupczak.org */

#ifdef WIN32
#include <windows.h>
#include <pthread.h>
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#include <strings.h>
#endif
#include <stdlib.h>

#include <openssl/ssl.h>

#include "xmp.h"
#include "socklib.h"
#include "xmp_session.h"

/* format of versionStr variable is "KorgVersion<space>%3.2f" 
   that way, our software distribution and updating component
   can correctly figure out version of some distributable
   binary thing */   

char *versionStr = "KorgVersion 1.00";

/* declare these variables for xmp and xmp_session */

xmpStats_t xmpStats;
int xmp_session_verbose = 0;

/* ************************************************************* */

int main(int argc, char *argv[])
{
     int ret;
     xmpSession_t *sp;
     u_int target;
     int i;
     char *authenUser;
     char *cptr;
     char fqdn[512];
     xmpPdu_t s,r;
     short xmpPort = XMP_PORT;
     xmpVariable_t var1,var2,var3, *varptr;
     char *filename;
     char *encodedStr, *decodedStr;
     int decodedLen;
     FILE *fp;
#ifdef WIN32
     WSADATA wsaData;

     if (WSAStartup(0x202,&wsaData) == SOCKET_ERROR) {
        perror("WSAStartup failed");
        WSACleanup();
        exit(-1);
     }
#endif

     if (argc < 4) {
        fprintf(stderr,"usage: %s host[:port] authenUser graphfile\n",argv[0]);
        exit(-1);
     }

     filename = argv[3];

     /* figure out our FQDN */
     get_fqdn(fqdn,511);

     DBG1("xmpgetsubgraph: using fqdn '%s'\n",fqdn);

     init_ssl_stuff(fqdn,"xmpgetsubgraph.pem","xmpgetsubgraph.csr");

     if ((cptr = strchr(argv[1],':')) != NULL) {
        *cptr = 0;
        cptr++;
        xmpPort = atoi(cptr);
     }

     if ((target = hostname_to_ipaddr(argv[1])) == 0) {
        fprintf(stderr,"Unknown host '%s'\n",argv[1]);
        exit(-1);
     }

     authenUser = argv[2];

     if ((sp = newXmpSessionConnect(target,htons(xmpPort),authenUser))== NULL) {
        fprintf(stderr,"Failed to connect to %s\n",argv[1]);
        exit(-1);
     }

     fprintf(stderr,"xmpgetsubgraph: connected to %s,%d\n",argv[1],xmpPort);
     DBG1("xmpgetsubgraph: wireHdr_t is %d\n",sizeof(xmpWireHdr_t));

     // construct query
     //  cartographer.numGraphNodes
     //  cartographer.numGraphEdges
     //  cartographer.subGraph

     bzero((void *)&s,sizeof(s));
     s.msgType = GetRequest;
     s.messageID = (int64_t)time(NULL);
     s.errorStatus = noError;
     s.errorIndex = 0;

     s.numberVariables = 3;
     s.varList = &var1;
     s.varListTail = &var3;
     bzero((void *)&var1,sizeof(var1));
     bzero((void *)&var2,sizeof(var2));
     bzero((void *)&var3,sizeof(var3));
     var1.next = &var2;
     var2.next = &var3;
     var3.next = NULL;

     sprintf(var1.varName.mibName,"cartographer");
     sprintf(var1.varName.objName,"numGraphNodes");
     var1.varSyntax = nullSyntax;
     var1.valueStrLen = 0; 

     sprintf(var2.varName.mibName,"cartographer");
     sprintf(var2.varName.objName,"numGraphEdges");
     var2.varSyntax = nullSyntax;
     var2.valueStrLen = 0; 

     sprintf(var3.varName.mibName,"cartographer");
     sprintf(var3.varName.objName,"subGraph");
     var3.varSyntax = nullSyntax;
     var3.valueStrLen = 0; 

     bzero((void *)&r,sizeof(r));

     if ((ret = xmpTransaction(sp,&s,&r)) < 0) {
        fprintf(stderr,"xmpgetsubgraph: transaction failed, %d\n",ret);
     }

     if ((r.errorStatus == noError) && (r.numberVariables == 3)) {

        varptr = r.varList;
        if (varptr != NULL) {

	   // numberNodes
	  if (strcmp(varptr->varName.objName,"numGraphNodes") == 0)
	     fprintf(stderr,"xmpgetsubgraph: nodes %s\n",varptr->valueStr);
           varptr = varptr->next;

           // numberEdges
	   if (strcmp(varptr->varName.objName,"numGraphEdges") == 0)
	      fprintf(stderr,"xmpgetsubgraph: edges %s\n",varptr->valueStr);

           varptr = varptr->next;

           // subgraph base64 encoded
           if (strcmp(varptr->varName.objName,"subGraph") == 0) {
	     DBG1("xmpgetsubgraph: subGraph encoded length %d\n",varptr->valueStrLen);

              // base64 decode
              if (varptr->valueBuf != NULL)
	 	 encodedStr = (char *)varptr->valueBuf;
	      else 
	 	 encodedStr = varptr->valueStr;

              decodedLen = 0;
              decodedStr = (char *)base64_decode((u_char *)encodedStr,
                                         (int)varptr->valueStrLen,
                                         (int *)&decodedLen);

              DBG2("xmpgetsubgraph: decoded str len is %d %d\n",
                   decodedLen, strlen(decodedStr));

              // write to file
              DBG1("xmpgetsubgraph: decoded length %d\n",decodedLen);

              if ((fp = fopen(filename,"w")) != NULL) {
	 	 fprintf(fp,"%s",decodedStr);
                 fclose(fp);
              }

           }

        }
     }


     destroyXmpSession(sp);
}

/* ************************************************************* */

